@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: FilePicker.h,v 1.8 2004/06/07 14:19:54 chpe Exp $
+ *  $Id: FilePicker.h,v 1.8.8.1 2004/11/13 19:56:49 chpe Exp $
  */
 
 #ifndef EPHY_FILEPICKER_H
@@ -24,7 +24,9 @@
 
 #include <nsIFilePicker.h>
 #include <nsISupports.h>
+#define MOZILLA_STRICT_API
 #include <nsEmbedString.h>
+#undef MOZILLA_STRICT_API
 
 #include "ephy-file-chooser.h"
 
