@@ -15,9 +15,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-arrow-toolbutton.c,v 1.8 2003/11/03 22:41:36 chpe Exp $
+ *  $Id: ephy-arrow-toolbutton.c,v 1.10 2004/03/17 09:02:37 marco Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "ephy-arrow-toolbutton.h"
 #include "ephy-marshal.h"
 #include "ephy-gui.h"
@@ -257,7 +261,7 @@ ephy_arrow_toolbutton_finalize (GObject *object)
 {
 	EphyArrowToolButton *arrow_toolbutton = EPHY_ARROW_TOOLBUTTON (object);
 
-	gtk_widget_destroy (GTK_WIDGET (arrow_toolbutton->priv->menu));
+	g_object_unref (arrow_toolbutton->priv->menu);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
