@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-main.c,v 1.24 2004/03/05 23:38:10 chpe Exp $
+ *  $Id: ephy-main.c,v 1.25.4.2 2005/02/01 19:41:29 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -83,6 +83,13 @@ shell_weak_notify (gpointer data,
 	gtk_main_quit ();
 }
 
+static gboolean
+idle_unref (GObject *object)
+{
+	g_object_unref (object);
+	return FALSE;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -106,7 +113,6 @@ main (int argc, char *argv[])
 	 */
 	LIBXML_TEST_VERSION
 
-	g_set_application_name (_("Epiphany Web Browser"));
 	program = gnome_program_init (PACKAGE, VERSION,
                                       LIBGNOMEUI_MODULE, argc, argv,
                                       GNOME_PARAM_POPT_TABLE, popt_options,
@@ -114,6 +120,8 @@ main (int argc, char *argv[])
 				      GNOME_PARAM_APP_DATADIR, DATADIR,
                                       NULL);
 
+	g_set_application_name (_("Epiphany Web Browser"));
+
         g_object_get_property (G_OBJECT (program),
                                GNOME_PARAM_POPT_CONTEXT,
                                g_value_init (&context_as_value, G_TYPE_POINTER));
@@ -182,11 +190,10 @@ main (int argc, char *argv[])
 						GTK_MESSAGE_ERROR, err->message);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 	}
-	else if (new_instance)
+	else if (new_instance && ephy_shell)
 	{
-		g_object_unref (ephy_shell);
-
 		g_object_weak_ref (G_OBJECT (ephy_shell), shell_weak_notify, NULL);
+		g_idle_add ((GSourceFunc) idle_unref, ephy_shell);
 
 		gtk_main ();
 	}
