@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.112.4.3 2004/11/12 21:40:27 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.112.4.6 2005/01/22 18:32:01 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -65,6 +65,8 @@
 
 #define CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME  "/apps/epiphany/lockdown/disable_javascript_chrome"
 
+#define MAX_HIDDEN_POPUPS 5
+
 struct _EphyTabPrivate
 {
 	char *status_message;
@@ -302,6 +304,14 @@ ephy_tab_map (GtkWidget *widget)
 	GTK_WIDGET_CLASS (parent_class)->map (widget);	
 }
 
+static void
+ephy_tab_grab_focus (GtkWidget *widget)
+{
+	EphyTab *tab = EPHY_TAB (widget);
+
+	ephy_embed_activate (ephy_tab_get_embed (tab));
+}
+
 static EphyWindow *
 ephy_tab_get_window (EphyTab *tab)
 {
@@ -344,6 +354,7 @@ ephy_tab_class_init (EphyTabClass *class)
 
 	widget_class->size_allocate = ephy_tab_size_allocate;
 	widget_class->map = ephy_tab_map;
+	widget_class->grab_focus = ephy_tab_grab_focus;
 
 	g_object_class_install_property (object_class,
 					 PROP_ADDRESS,
@@ -437,7 +448,7 @@ ephy_tab_class_init (EphyTabClass *class)
 
 	g_object_class_install_property (object_class,
 					 PROP_VISIBLE,
-					 g_param_spec_boolean ("visible",
+					 g_param_spec_boolean ("visibility",
 							       "Visibility",
 							       "The tab's visibility",
 							       TRUE,
@@ -484,7 +495,25 @@ popups_manager_add (EphyTab *tab,
 	tab->priv->hidden_popups = g_slist_prepend
 		(tab->priv->hidden_popups, popup);
 
-	g_object_notify (G_OBJECT (tab), "hidden-popup-count");
+	if (popup_blocker_n_hidden (tab) > MAX_HIDDEN_POPUPS) /* bug #160863 */
+	{
+		/* Remove the oldest popup */
+		GSList *l = tab->priv->hidden_popups;
+
+		while (l->next->next != NULL)
+		{
+			l = l->next;
+		}
+
+		popup = (PopupInfo *) l->next->data;
+		popups_manager_free_info (popup);
+
+		l->next = NULL;
+	}
+	else
+	{
+		g_object_notify (G_OBJECT (tab), "hidden-popup-count");
+	}
 }
 
 static gboolean
@@ -1332,7 +1361,7 @@ ephy_tab_visibility_cb (EphyEmbed *embed, gboolean visibility,
 
 	tab->priv->visibility = visibility;
 
-	g_object_notify (G_OBJECT (tab), "visible");
+	g_object_notify (G_OBJECT (tab), "visibility");
 }
 
 static void
