@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.h,v 1.17 2004/03/23 21:17:12 chpe Exp $
+ *  $Id: ephy-tab.h,v 1.17.10.1 2004/10/08 21:12:09 chpe Exp $
  */
 
 #ifndef EPHY_TAB_H
@@ -35,9 +35,9 @@ G_BEGIN_DECLS
 #define EPHY_IS_TAB_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_TAB))
 #define EPHY_TAB_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_TAB, EphyTabClass))
 
-typedef struct EphyTabClass	EphyTabClass;
+typedef struct _EphyTabClass	EphyTabClass;
 typedef struct _EphyTab		EphyTab;
-typedef struct EphyTabPrivate	EphyTabPrivate;
+typedef struct _EphyTabPrivate	EphyTabPrivate;
 
 typedef enum
 {
@@ -61,14 +61,11 @@ struct _EphyTab
 	EphyTabPrivate *priv;
 };
 
-struct EphyTabClass
+struct _EphyTabClass
 {
 	GtkBinClass parent_class;
 };
 
-/* Include the header down here to resolve circular dependency */
-#include "ephy-window.h"
-
 GType			ephy_tab_get_type		(void);
 
 EphyTab	               *ephy_tab_new			(void);
