@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.206 2004/08/28 10:19:44 chpe Exp $
+ *  $Id: ephy-window.c,v 1.206.4.27 2005/04/04 21:03:36 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,6 +52,7 @@
 #include "ephy-stock-icons.h"
 #include "ephy-extension.h"
 #include "ephy-favicon-cache.h"
+#include "ephy-gui.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
@@ -320,16 +321,8 @@ static guint ephy_popups_n_entries = G_N_ELEMENTS (ephy_popups_entries);
 
 #define EPHY_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_WINDOW, EphyWindowPrivate))
 
-typedef enum
+struct _EphyWindowPrivate
 {
-	EPHY_WINDOW_MODE_NORMAL,
-	EPHY_WINDOW_MODE_FULLSCREEN,
-	EPHY_WINDOW_MODE_PRINT_PREVIEW
-} EphyWindowMode;
-
-struct EphyWindowPrivate
-{
-	EphyWindowMode mode;
 	GtkWidget *main_vbox;
 	GtkWidget *menu_dock;
 	GtkWidget *exit_fullscreen_popup;
@@ -346,14 +339,10 @@ struct EphyWindowPrivate
 	GtkNotebook *notebook;
 	EphyTab *active_tab;
 	EphyDialog *find_dialog;
-	gboolean closing;
-	gboolean has_size;
 	guint num_tabs;
 	guint tab_message_cid;
 	guint help_message_cid;
-
 	EphyEmbedChrome chrome;
-	gboolean should_save_chrome;
 
 	guint disable_arbitrary_url_notifier_id;
 	guint disable_bookmark_editing_notifier_id;
@@ -365,6 +354,12 @@ struct EphyWindowPrivate
 	guint disable_command_line_notifier_id;
 	guint browse_with_caret_notifier_id;
 	guint allow_popups_notifier_id;
+
+	guint closing : 1;
+	guint has_size : 1;
+	guint fullscreen_mode : 1;
+	guint ppv_mode : 1;
+	guint should_save_chrome : 1;
 };
 
 enum
@@ -457,29 +452,6 @@ destroy_exit_fullscreen_popup (EphyWindow *window)
 	}
 }
 
-static void
-ephy_window_destroy (GtkObject *gtkobject)
-{
-	EphyWindow *window = EPHY_WINDOW (gtkobject);
-
-	LOG ("EphyWindow destroy %p", window)
-
-	if (window->priv->closing == FALSE)
-	{
-		EphyExtension *manager;
-
-		window->priv->closing = TRUE;
-
-		/* Let the extensions detach themselves from the window */
-		manager = EPHY_EXTENSION (ephy_shell_get_extensions_manager (ephy_shell));
-		ephy_extension_detach_window (manager, window);
-	}
-
-	destroy_exit_fullscreen_popup (window);
-
-        GTK_OBJECT_CLASS (parent_class)->destroy (gtkobject);
-}
-
 static void
 add_widget (GtkUIManager *merge, GtkWidget *widget, EphyWindow *window)
 {
@@ -505,30 +477,33 @@ get_chromes_visibility (EphyWindow *window, gboolean *show_menubar,
 {
 	EphyEmbedChrome flags = window->priv->chrome;
 
-	switch (window->priv->mode)
+	if (window->priv->ppv_mode)
 	{
-		case EPHY_WINDOW_MODE_NORMAL:
-			*show_menubar = (flags & EPHY_EMBED_CHROME_MENUBAR) != 0;
-			*show_statusbar = (flags & EPHY_EMBED_CHROME_STATUSBAR) != 0;
-			*show_toolbar = (flags & EPHY_EMBED_CHROME_TOOLBAR) != 0;
-			*show_bookmarksbar = (flags & EPHY_EMBED_CHROME_BOOKMARKSBAR) != 0;
-			break;
-		case EPHY_WINDOW_MODE_FULLSCREEN:
-			*show_toolbar = (flags & EPHY_EMBED_CHROME_TOOLBAR) != 0;
-			*show_menubar = *show_statusbar = *show_bookmarksbar = FALSE;
-			break;
-		default:
-			*show_menubar = *show_statusbar = *show_toolbar = *show_bookmarksbar = FALSE;
-			break;
+		*show_menubar = *show_statusbar = *show_toolbar = *show_bookmarksbar = FALSE;
+	}
+	else if (window->priv->fullscreen_mode)
+	{
+		*show_toolbar = (flags & EPHY_EMBED_CHROME_TOOLBAR) != 0;
+		*show_menubar = *show_statusbar = *show_bookmarksbar = FALSE;
+	}
+	else
+	{
+		*show_menubar = (flags & EPHY_EMBED_CHROME_MENUBAR) != 0;
+		*show_statusbar = (flags & EPHY_EMBED_CHROME_STATUSBAR) != 0;
+		*show_toolbar = (flags & EPHY_EMBED_CHROME_TOOLBAR) != 0;
+		*show_bookmarksbar = (flags & EPHY_EMBED_CHROME_BOOKMARKSBAR) != 0;
 	}
 }
 
 static void
 sync_chromes_visibility (EphyWindow *window)
 {
+	EphyWindowPrivate *priv = window->priv;
 	GtkWidget *menubar;
 	gboolean show_statusbar, show_menubar, show_toolbar, show_bookmarksbar;
 
+	if (priv->closing) return;
+
 	get_chromes_visibility (window, &show_menubar,
 				&show_statusbar, &show_toolbar,
 				&show_bookmarksbar);
@@ -548,12 +523,17 @@ ephy_window_fullscreen (EphyWindow *window)
 {
 	GtkWidget *popup, *button, *icon, *label, *hbox;
 
-	window->priv->mode = EPHY_WINDOW_MODE_FULLSCREEN;
+	window->priv->fullscreen_mode = TRUE;
 
 	sync_chromes_visibility (window);
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_FULLSCREEN))
 	{
+		egg_editable_toolbar_set_model
+			(EGG_EDITABLE_TOOLBAR (window->priv->toolbar),
+			 EGG_TOOLBARS_MODEL (
+				ephy_shell_get_toolbars_model (ephy_shell, TRUE)));
+
 		/* no need to show "exit fullscreen" button */
 		return;
 	}
@@ -599,7 +579,7 @@ ephy_window_fullscreen (EphyWindow *window)
 static void
 ephy_window_unfullscreen (EphyWindow *window)
 {
-	window->priv->mode = EPHY_WINDOW_MODE_NORMAL;
+	window->priv->fullscreen_mode = FALSE;
 
 	destroy_exit_fullscreen_popup (window);
 
@@ -661,6 +641,8 @@ confirm_close_with_modified_forms (EphyWindow *window)
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					      _("Close _Document"), GTK_RESPONSE_OK,
 					      NULL);
+
+	ephy_gui_set_default_window_icon (GTK_WINDOW (dialog));
 	
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
@@ -722,8 +704,13 @@ ephy_window_key_press_event (GtkWidget *widget,
 	GtkWidget *menubar;
 	guint modifiers = gtk_accelerator_get_default_mod_mask ();
 
-	/* Show and activate the menubar on F10, if it isn't visible */
-        if (event->keyval == GDK_F10 && (event->state & modifiers) == 0)
+	/* Show and activate the menubar on F10, if it isn't visible,
+	 * unless we're in ppv or lockdown mode
+	 */
+        if (window->priv->ppv_mode == FALSE &&
+	    !eel_gconf_get_boolean (CONF_LOCKDOWN_HIDE_MENUBAR) && 
+	    event->keyval == GDK_F10 &&
+	    (event->state & modifiers) == 0)
 	{
 		menubar = gtk_ui_manager_get_widget
 			(GTK_UI_MANAGER (window->ui_merge), "/menubar");
@@ -754,7 +741,7 @@ ephy_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, EphyWindow *win
 	/* Workaround a crash when closing a window while in print preview mode. See
 	 * mozilla bug #241809
 	 */
-	if (window->priv->mode == EPHY_WINDOW_MODE_PRINT_PREVIEW)
+	if (window->priv->ppv_mode)
 	{
 		EphyEmbed *embed;
 
@@ -1002,14 +989,34 @@ update_chromes_actions (EphyWindow *window)
 		 			   window);
 }
 
+static void
+update_print_actions (EphyWindow *window,
+		      gboolean enable)
+{
+	GtkActionGroup *action_group = window->priv->action_group;
+	GtkAction *action;
+	gboolean printing, print_setup;
+
+	printing = !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINTING);
+	print_setup = !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINT_SETUP) &&
+		!eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_COMMAND_LINE);
+
+	action = gtk_action_group_get_action (action_group, "FilePrintSetup");
+	g_object_set (action, "sensitive", printing && print_setup, NULL);
+	action = gtk_action_group_get_action (action_group, "FilePrintPreview");
+	g_object_set (action, "sensitive", enable && printing, NULL);
+	action = gtk_action_group_get_action (action_group, "FilePrint");
+	g_object_set (action, "sensitive", enable && printing, NULL);
+
+}
+
 static void
 update_actions_sensitivity (EphyWindow *window)
 {
 	GtkActionGroup *action_group = window->priv->action_group;
 	GtkActionGroup *popups_action_group = window->priv->popups_action_group;
 	GtkAction *action;
-	gboolean bookmarks_editable, save_to_disk;
-	gboolean printing, print_setup, fullscreen;
+	gboolean bookmarks_editable, save_to_disk, fullscreen;
 
 	action = gtk_action_group_get_action (action_group, "ViewToolbar");
 	g_object_set (action, "sensitive", eel_gconf_key_is_writable (CONF_WINDOWS_SHOW_TOOLBARS), NULL);
@@ -1043,23 +1050,14 @@ update_actions_sensitivity (EphyWindow *window)
 	g_object_set (action, "sensitive", save_to_disk, NULL);
 	action = gtk_action_group_get_action (popups_action_group, "DownloadLink");
 	g_object_set (action, "sensitive", save_to_disk, NULL);
+	action = gtk_action_group_get_action (popups_action_group, "DownloadLinkAs");
+	g_object_set (action, "sensitive", save_to_disk, NULL);
 	action = gtk_action_group_get_action (popups_action_group, "SaveBackgroundAs");
 	g_object_set (action, "sensitive", save_to_disk, NULL);
 	action = gtk_action_group_get_action (popups_action_group, "SaveImageAs");
 	g_object_set (action, "sensitive", save_to_disk, NULL);
-
-	printing = !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINTING);
-	print_setup = !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINT_SETUP) &&
-		!eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_COMMAND_LINE);
-	action = gtk_action_group_get_action (action_group, "FilePrintSetup");
-	g_object_set (action, "sensitive", printing && print_setup, NULL);
-	action = gtk_action_group_get_action (action_group, "FilePrintPreview");
-	g_object_set (action, "sensitive", printing && print_setup, NULL);
-	action = gtk_action_group_get_action (action_group, "FilePrint");
-	g_object_set (action, "sensitive", printing, NULL);
-
 	action = gtk_action_group_get_action (popups_action_group, "SetImageAsBackground");
-	g_object_set (action, "sensitive", eel_gconf_key_is_writable (CONF_DESKTOP_BG_PICTURE), NULL);
+	g_object_set (action, "sensitive", save_to_disk && eel_gconf_key_is_writable (CONF_DESKTOP_BG_PICTURE), NULL);
 
 	action = gtk_action_group_get_action (action_group, "EditToolbar");
 	g_object_set (action, "sensitive", ! eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_TOOLBAR_EDITING), NULL);
@@ -1071,6 +1069,8 @@ update_actions_sensitivity (EphyWindow *window)
 	g_object_set (action, "sensitive", !fullscreen, NULL);
 	action = gtk_action_group_get_action (action_group, "TabsDetach");
 	g_object_set (action, "sensitive", !fullscreen, NULL);
+
+	update_print_actions (window, TRUE);
 }
 
 static void
@@ -1413,6 +1413,9 @@ sync_tab_load_status (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 	status = ephy_tab_get_load_status (tab);
 	g_object_set (action, "sensitive", status, NULL);
 
+	/* disable print while loading, see bug #116344 */
+	update_print_actions (window, !status);
+
 	if (status)
 	{
 		toolbar_spinner_start (window->priv->toolbar);
@@ -1527,18 +1530,28 @@ static void
 popup_menu_at_coords (GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
 		      gpointer user_data)
 {
-	EphyEmbedEvent *event = (EphyEmbedEvent *) user_data;
+	GtkWidget *window = GTK_WIDGET (user_data);
+	EphyEmbedEvent *event;
+
+	event = g_object_get_data (G_OBJECT (window), "context_event");
+	g_return_if_fail (event != NULL);
 
 	ephy_embed_event_get_coords (event, x, y);
 
+	/* FIXME: better position the popup within the window bounds? */
+	ephy_gui_sanitise_popup_position (menu, window, x, y);
+
 	*push_in = TRUE;
 }
 
 static void
 hide_embed_popup_cb (GtkWidget *popup,
-			EphyWindow *window)
+		     EphyWindow *window)
 {
 	enable_edit_actions_sensitivity (window);
+
+	g_signal_handlers_disconnect_by_func
+		(popup, G_CALLBACK (hide_embed_popup_cb), window);
 }
 
 static void
@@ -1554,7 +1567,7 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	EphyEmbedEventType type;
 
 	/* Do not show the menu in print preview mode */
-	if (window->priv->mode == EPHY_WINDOW_MODE_PRINT_PREVIEW)
+	if (window->priv->ppv_mode)
 	{
 		return;
 	}
@@ -1619,8 +1632,9 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	if (type == EPHY_EMBED_EVENT_KEY)
 	{
 		gtk_menu_popup (GTK_MENU (widget), NULL, NULL,
-				popup_menu_at_coords, event, 2,
+				popup_menu_at_coords, window, 0,
 				gtk_get_current_event_time ());
+		gtk_menu_shell_select_first (GTK_MENU_SHELL (widget), FALSE);
 	}
 	else
 	{
@@ -1813,19 +1827,53 @@ update_tabs_menu_sensitivity (EphyWindow *window)
 	g_object_set (action, "sensitive", detach, NULL);
 }
 
+static gboolean
+modal_alert_cb (EphyEmbed *embed,
+		EphyWindow *window)
+{
+	EphyTab *tab;
+	char *address;
+
+	tab = ephy_tab_for_embed (embed);
+	g_return_val_if_fail (tab != NULL, FALSE);
+
+	/* switch the window to the tab, and bring the window to the foreground
+	 * (since the alert is modal, the user won't be able to do anything
+	 * with his current window anyway :|)
+	 */
+	ephy_window_jump_to_tab (window, tab);
+	gtk_window_present (GTK_WINDOW (window));
+
+	/* make sure the location entry shows the real URL of the tab's page */
+	address = ephy_embed_get_location (embed, TRUE);
+	toolbar_set_location (window->priv->toolbar, address);
+	g_free (address);
+
+	/* don't suppress alert */
+	return FALSE;
+}
+
 static void
 tab_added_cb (EphyNotebook *notebook,
 	      EphyTab *tab,
 	      EphyWindow *window)
 {
+	EphyEmbed *embed;
+
         g_return_if_fail (EPHY_IS_TAB (tab));
 
 	window->priv->num_tabs++;
 
 	update_tabs_menu_sensitivity (window);
 
-	g_signal_connect_object (G_OBJECT (tab), "notify::visible",
+	g_signal_connect_object (G_OBJECT (tab), "notify::visibility",
 				 G_CALLBACK (sync_tab_visibility), window, 0);
+
+	embed = ephy_tab_get_embed (tab);
+	g_return_if_fail (embed != NULL);
+
+	g_signal_connect_after (embed, "ge-modal-alert",
+				G_CALLBACK (modal_alert_cb), window);
 }
 
 static void
@@ -1833,6 +1881,8 @@ tab_removed_cb (EphyNotebook *notebook,
 		EphyTab *tab,
 		EphyWindow *window)
 {
+	EphyEmbed *embed;
+
         g_return_if_fail (EPHY_IS_TAB (tab));
 
 	g_signal_handlers_disconnect_by_func (G_OBJECT (tab),
@@ -1845,6 +1895,12 @@ tab_removed_cb (EphyNotebook *notebook,
 	{
 		update_tabs_menu_sensitivity (window);
 	}
+
+	embed = ephy_tab_get_embed (tab);
+	g_return_if_fail (embed != NULL);
+
+	g_signal_handlers_disconnect_by_func
+		(embed, G_CALLBACK (modal_alert_cb), window);
 }
 
 static void
@@ -1956,6 +2012,83 @@ ephy_window_set_chrome (EphyWindow *window, EphyEmbedChrome mask)
 	update_chromes_actions (window);
 }
 
+static void
+ephy_window_dispose (GObject *object)
+{
+	EphyWindow *window = EPHY_WINDOW (object);
+	EphyWindowPrivate *priv = window->priv;
+	GObject *single;
+	GSList *popups;
+
+	LOG ("EphyWindow dispose %p", window);
+
+	/* Only do these once */
+	if (window->priv->closing == FALSE)
+	{
+		EphyExtension *manager;
+
+		window->priv->closing = TRUE;
+
+		/* Let the extensions detach themselves from the window */
+		manager = EPHY_EXTENSION (ephy_shell_get_extensions_manager (ephy_shell));
+		ephy_extension_detach_window (manager, window);
+
+		/* Deactivate menus */
+		popups = gtk_ui_manager_get_toplevels (GTK_UI_MANAGER (window->ui_merge), GTK_UI_MANAGER_POPUP);
+		g_slist_foreach (popups, (GFunc) gtk_menu_shell_deactivate, NULL);
+		g_slist_free (popups);
+	
+		single = ephy_embed_shell_get_embed_single (embed_shell);
+		g_signal_handlers_disconnect_by_func
+			(single, G_CALLBACK (network_status_changed), window);
+	
+		eel_gconf_notification_remove (priv->disable_arbitrary_url_notifier_id);
+		eel_gconf_notification_remove (priv->disable_bookmark_editing_notifier_id);
+		eel_gconf_notification_remove (priv->disable_toolbar_editing_notifier_id);
+		eel_gconf_notification_remove (priv->disable_history_notifier_id);
+		eel_gconf_notification_remove (priv->disable_printing_notifier_id);
+		eel_gconf_notification_remove (priv->disable_print_setup_notifier_id);
+		eel_gconf_notification_remove (priv->disable_save_to_disk_notifier_id);
+		eel_gconf_notification_remove (priv->disable_command_line_notifier_id);
+		eel_gconf_notification_remove (priv->browse_with_caret_notifier_id);
+		eel_gconf_notification_remove (priv->allow_popups_notifier_id);
+
+		if (priv->find_dialog)
+		{
+			g_object_unref (G_OBJECT (priv->find_dialog));
+			priv->find_dialog = NULL;
+		}
+	
+		g_object_unref (priv->fav_menu);
+		priv->fav_menu = NULL;
+
+		g_object_unref (priv->enc_menu);
+		priv->enc_menu = NULL;
+
+		g_object_unref (priv->tabs_menu);
+		priv->tabs_menu = NULL;
+
+		g_object_unref (priv->bmk_menu);
+		priv->bmk_menu = NULL;
+	
+		if (priv->ppview_toolbar)
+		{
+			g_object_unref (priv->ppview_toolbar);
+			priv->ppview_toolbar = NULL;
+		}
+	
+		g_object_unref (priv->action_group);
+		priv->action_group = NULL;
+
+		g_object_unref (window->ui_merge);
+		window->ui_merge = NULL;
+	}
+
+	destroy_exit_fullscreen_popup (window);
+
+        G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 static void
 ephy_window_set_property (GObject *object,
 			  guint prop_id,
@@ -1998,17 +2131,15 @@ static void
 ephy_window_class_init (EphyWindowClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GtkObjectClass *gtkobject_class = GTK_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
         parent_class = g_type_class_peek_parent (klass);
 
+	object_class->dispose = ephy_window_dispose;
         object_class->finalize = ephy_window_finalize;
 	object_class->get_property = ephy_window_get_property;
 	object_class->set_property = ephy_window_set_property;
 
-	gtkobject_class->destroy = ephy_window_destroy;
-
 	widget_class->show = ephy_window_show;
 	widget_class->key_press_event = ephy_window_key_press_event;
 
@@ -2158,7 +2289,8 @@ ephy_window_init (EphyWindow *window)
 	window->priv->has_size = FALSE;
 	window->priv->chrome = EPHY_EMBED_CHROME_ALL;
 	window->priv->should_save_chrome = FALSE;
-	window->priv->mode = EPHY_WINDOW_MODE_NORMAL;
+	window->priv->fullscreen_mode = FALSE;
+	window->priv->ppv_mode = FALSE;
 
 	ensure_default_icon ();
 
@@ -2199,6 +2331,9 @@ ephy_window_init (EphyWindow *window)
 	window->priv->toolbar = toolbar_new (window);
 	window->priv->bookmarksbar = ephy_bookmarksbar_new (window);
 
+	g_signal_connect_swapped (window->priv->toolbar, "activation-finished",
+				  G_CALLBACK (sync_chromes_visibility), window);
+
 	/* forward the toolbar's action_request signal to the bookmarks toolbar,
 	 * so the user can also have bookmarks on the normal toolbar
 	 */
@@ -2299,45 +2434,9 @@ ephy_window_init (EphyWindow *window)
 static void
 ephy_window_finalize (GObject *object)
 {
-        EphyWindow *window = EPHY_WINDOW (object);
-	GObject *single;
-
-	single = ephy_embed_shell_get_embed_single (embed_shell);
-	g_signal_handlers_disconnect_by_func
-		(single, G_CALLBACK (network_status_changed), window);
-
-	eel_gconf_notification_remove (window->priv->disable_arbitrary_url_notifier_id);
-	eel_gconf_notification_remove (window->priv->disable_bookmark_editing_notifier_id);
-	eel_gconf_notification_remove (window->priv->disable_toolbar_editing_notifier_id);
-	eel_gconf_notification_remove (window->priv->disable_history_notifier_id);
-	eel_gconf_notification_remove (window->priv->disable_printing_notifier_id);
-	eel_gconf_notification_remove (window->priv->disable_print_setup_notifier_id);
-	eel_gconf_notification_remove (window->priv->disable_save_to_disk_notifier_id);
-	eel_gconf_notification_remove (window->priv->disable_command_line_notifier_id);
-	eel_gconf_notification_remove (window->priv->browse_with_caret_notifier_id);
-	eel_gconf_notification_remove (window->priv->allow_popups_notifier_id);
-
-	if (window->priv->find_dialog)
-	{
-		g_object_unref (G_OBJECT (window->priv->find_dialog));
-	}
-
-	g_object_unref (window->priv->fav_menu);
-	g_object_unref (window->priv->enc_menu);
-	g_object_unref (window->priv->tabs_menu);
-	g_object_unref (window->priv->bmk_menu);
-
-	if (window->priv->ppview_toolbar)
-	{
-		g_object_unref (window->priv->ppview_toolbar);
-	}
-
-	g_object_unref (window->priv->action_group);
-	g_object_unref (window->ui_merge);
-
         G_OBJECT_CLASS (parent_class)->finalize (object);
 
-	LOG ("Ephy Window finalized %p", window)
+	LOG ("Ephy Window finalized %p", object)
 
 	g_object_unref (ephy_shell);
 }
@@ -2383,16 +2482,12 @@ void
 ephy_window_set_print_preview (EphyWindow *window, gboolean enabled)
 {
 	GtkAccelGroup *accel_group;
-	EphyWindowMode mode;
 
 	accel_group = gtk_ui_manager_get_accel_group (GTK_UI_MANAGER (window->ui_merge));
 
-	mode = enabled ? EPHY_WINDOW_MODE_PRINT_PREVIEW :
-			 EPHY_WINDOW_MODE_NORMAL;
+	if (window->priv->ppv_mode == enabled) return;
 
-	if (mode == window->priv->mode) return;
-
-	window->priv->mode = mode;
+	window->priv->ppv_mode = enabled;
 
 	sync_chromes_visibility (window);
 	ephy_notebook_set_show_tabs (EPHY_NOTEBOOK (window->priv->notebook), !enabled);
@@ -2592,7 +2687,15 @@ ephy_window_load_url (EphyWindow *window,
         g_return_if_fail (url != NULL);
 
         ephy_embed_load_url (embed, url);
-	ephy_embed_activate (embed);
+
+	if (url == NULL || url[0] == '\0' || strcmp (url, "about:blank") == 0)
+	{
+		toolbar_activate_location (window->priv->toolbar);
+	}
+	else
+	{
+		ephy_embed_activate (embed);
+	}
 }
 
 /**
@@ -2613,7 +2716,7 @@ ephy_window_activate_location (EphyWindow *window)
  *
  * Shows @widget if it is hidden.
  **/
-void
+static void
 ephy_window_show (GtkWidget *widget)
 {
 	EphyWindow *window = EPHY_WINDOW(widget);
@@ -2630,7 +2733,7 @@ ephy_window_show (GtkWidget *widget)
 		if (width == -1 && height == -1)
 		{
 			ephy_state_add_window (widget, "main_window", 600, 500,
-					       EPHY_STATE_WINDOW_SAVE_SIZE);
+					       TRUE, EPHY_STATE_WINDOW_SAVE_SIZE);
 		}
 
 		window->priv->has_size = TRUE;
@@ -2651,7 +2754,6 @@ EphyTab *
 ephy_window_get_active_tab (EphyWindow *window)
 {
 	g_return_val_if_fail (EPHY_IS_WINDOW (window), NULL);
-	g_return_val_if_fail (EPHY_IS_TAB (window->priv->active_tab), NULL);
 
 	return window->priv->active_tab;
 }
